(* ========================================================================= *)
(*                                                                           *)
(*               Formalization of Composed Optical Systems                   *)
(*                                                                           *)
(*        (c) Copyright, Umair Siddique, 2012-2015                           *)
(*                       Hardware Verification Group (HVG),                  *)
(*                       Concordia University                                *)
(*                                                                           *)
(*     Contact: <muh_sidd@ece.concordia.ca>                                  *)
(*                                                                           *)
(* Last update: September 10,2015                                            *)
(*                                                                           *)
(* ========================================================================= *)

needs "ray_optics.ml";;

new_type_abbrev("composed_system",`:optical_system list`);;

let composed_system = define 
   `composed_system ([]:optical_system list)  = 
              cmat2x2 (Cx(&1)) (Cx(&0))  (Cx(&0)) (Cx(&1)) /\
    composed_system ((CONS sys) cs) = 
    composed_system cs ** system_composition sys `;;


let is_valid_composed_system = new_definition
   `is_valid_composed_system  (sys:optical_system list) <=>
    ALL is_valid_optical_system sys `;;

new_type_abbrev("gray",`:ray list`);;

let is_valid_genray = define `
    (is_valid_genray ([]:gray) <=> F) /\ 
     (is_valid_genray (CONS h t) <=> 
     ((last_single_ray h = fst_single_ray (HD t)) /\ 
      is_valid_genray t ))`;;

   
let is_valid_gray_in_system = define `
  (is_valid_gray_in_system (cray:gray) ([]:optical_system list) <=> F) /\ 
  (is_valid_gray_in_system  [] (sys:optical_system list) <=> F) /\ 
  (is_valid_gray_in_system (CONS r1 rs) (CONS h hs) <=> 
        ((is_valid_ray_in_system r1 h) /\ (is_valid_gray_in_system rs hs)))`;;


let system_of_system  = composed_system ;;

let is_valid_sys_of_sys = is_valid_composed_system;; 

let is_valid_cray_in_system = is_valid_gray_in_system;;


let COMPOSED_SYSTEM_MATRIX = prove(`
   !sys cray.
   is_valid_composed_system (sys:optical_system list) /\ 
   is_valid_genray (cray:gray) /\
   is_valid_gray_in_system (cray:gray) sys ==> 
      let (y0,theta0) = fst_single_ray (HD (cray)) and
          (y_n,theta_n) = last_single_ray (LAST (cray)) in
   vector [Cx y_n;Cx theta_n] = composed_system sys ** vector [Cx y0;Cx theta0]`,

LIST_INDUCT_TAC THENL[
REWRITE_TAC[FORALL_RAY_THM;FORALL_SINGLE_RAY_THM; is_valid_cray_in_system];
ALL_TAC] THEN
REWRITE_TAC[FORALL_RAY_THM;FORALL_SINGLE_RAY_THM;ALL;is_valid_sys_of_sys; 
            system_of_system;GSYM CMATRIX_VECTOR_MUL_ASSOC] THEN
REPEAT(LIST_INDUCT_TAC ORELSE GEN_TAC) THEN 
REWRITE_TAC[FORALL_RAY_THM;FORALL_SINGLE_RAY_THM; 
            is_valid_genray;is_valid_cray_in_system] THEN
REWRITE_TAC[FORALL_RAY_THM;FORALL_SINGLE_RAY_THM;is_valid_genray; 
            is_valid_cray_in_system] THEN
REWRITE_TAC[FORALL_RAY_THM;FORALL_SINGLE_RAY_THM;ALL;is_valid_sys_of_sys;
              system_of_system;GSYM CMATRIX_VECTOR_MUL_ASSOC] THEN
REPEAT STRIP_TAC THEN 
MP_TAC SYSTEM_MATRIX THEN

DISCH_THEN(MP_TAC o SPEC `h:optical_system`) THEN
DISCH_THEN(MP_TAC o SPEC `h':ray`) THEN 
ANTS_TAC THENL[
ASM_SIMP_TAC[];ALL_TAC] THEN

REPEAT (LET_TAC) THEN
REPEAT(POP_ASSUM MP_TAC) THEN
REWRITE_TAC[PAIR_EQ;fst_single_ray;HD] THEN
ONCE_REWRITE_TAC[EQ_SYM_EQ] THEN
REPEAT STRIP_TAC THEN
POP_ASSUM MP_TAC THEN
ONCE_ASM_REWRITE_TAC[] THEN
SIMP_TAC[]THEN 
REPEAT STRIP_TAC THEN
EVERY_ASSUM (REPEAT_GTCL IMP_RES_THEN MP_TAC o REWRITE_RULE[]) THEN
DISCH_THEN(MP_TAC o SPEC `t':gray`) THEN
ANTS_TAC THENL[
ASM_SIMP_TAC[is_valid_sys_of_sys];ALL_TAC] THEN
REPEAT (LET_TAC) THEN
ONCE_REWRITE_TAC[PAIR_EQ] THEN
DISCH_TAC THEN
ONCE_REWRITE_TAC[EQ_SYM_EQ] THEN
REPEAT STRIP_TAC THEN
POP_ASSUM MP_TAC THEN
ONCE_REWRITE_TAC[EQ_SYM_EQ] THEN
REPEAT STRIP_TAC THEN
SUBGOAL_THEN ` (vector [Cx y_n';  Cx theta_n']:complex^2) = 
                vector [Cx y0'''; Cx theta0'''] ` ASSUME_TAC THENL[
ASM_SIMP_TAC[];ALL_TAC] THEN
ONCE_ASM_REWRITE_TAC[] THEN
ONCE_ASM_REWRITE_TAC[] THEN
SUBGOAL_THEN `last_single_ray (LAST (CONS ((y0',theta0'),(y1,theta1),rs) t')) =
               last_single_ray (LAST t') ` ASSUME_TAC THENL[
REPEAT(POP_ASSUM MP_TAC) THEN
ASM_CASES_TAC `t' :((real#real)#(real#real)#((real#real)#real#real)list)list = 
               [] ` THENL[
ONCE_ASM_REWRITE_TAC[ ] THEN 
REPEAT STRIP_TAC THEN
SUBGOAL_THEN ` is_valid_genray []` ASSUME_TAC THENL[
ONCE_ASM_REWRITE_TAC[ ];ALL_TAC] THEN
POP_ASSUM MP_TAC THEN
REWRITE_TAC[is_valid_genray ];ALL_TAC] THEN
ASM_REWRITE_TAC[LAST];ALL_TAC] THEN
POP_ASSUM MP_TAC THEN
ONCE_ASM_REWRITE_TAC[ ] THEN
REWRITE_TAC[PAIR_EQ] THEN
MESON_TAC[EQ_SYM_EQ]);;

























