(* ========================================================================= *)
(*                                                                           *)
(*                  Fabry Perot Cavity with fiber rod lens (FRL)             *)
(*                                                                           *)
(*        (c) Copyright, Umair Siddique, Vincent Aravantinos, 2012-2013      *)
(*                       Umair Siddique 2014-2015                            *)
(*                       Hardware Verification Group,                        *)
(*                       Concordia University                                *)
(*                                                                           *)
(*           Contact: <umair.hvg@gmail.com>                                  *)
(*                                                                           *)
(* Last update: Feb 2015                                                     *)
(*                                                                           *)
(* ========================================================================= *)


needs "resonator.ml";;

let asm_real_prove = REPEAT(POP_ASSUM MP_TAC) THEN CONV_TAC REAL_FIELD;;

(*----XZ--dimension--------*)

let fp_frl_xz_cavity = new_definition
  `fp_frl_xz_cavity R dx df nf :resonator  =
  (spherical_r R),[(&1,dx),plane_t;(nf,df),plane_t],
                 (&1,dx), (spherical_r R)`;;


let fp_frl_matrix_xz = new_definition 
  `fp_frl_matrix_xz R dx df nf = 
  mat2x2 (&1)  (df/nf + &2*dx) 
  (-- &2/R) (&1 - (&4*(dx))/R - (&2 *(df))/(R *(nf)))`;;


let VALID_FP_FRL_XZ_CAVITY = prove
  (`!R dx df nf.
    ~(R= &0) /\ &0 < dx /\ &0 < df /\ &0 < nf  ==>
      is_valid_resonator  (fp_frl_xz_cavity R dx df nf)`,
   VALID_RESONATOR_TAC fp_frl_xz_cavity);;


let FP_FRL_STABLE_XZ = prove( 
      `!R dx df nf.  ~(R= &0) /\ &0 < dx /\ &0 < df /\ &0 < nf /\  
      &0 < (&2*dx + df/nf)/R /\ (&2*dx + df/nf)/R < &2 ==> 
      is_stable_resonator (fp_frl_xz_cavity R dx df nf)`,

REPEAT STRIP_TAC THEN MP_REWRITE_TAC STABILITY_THEOREM_SYM THEN 
EXISTS_TAC(`fp_frl_matrix_xz R dx df nf  :real^2^2`) THEN 
CONJ_TAC THENL[ASM_SIMP_TAC[VALID_FP_FRL_XZ_CAVITY]; ALL_TAC] THEN
CONJ_TAC THENL[
REWRITE_TAC[fp_frl_matrix_xz;MAT_POW2] THEN
REWRITE_TAC[system_composition;unfold_resonator_s;LIST_POW;ONE;fp_frl_xz_cavity;
round_trip;APPEND;optical_components_shift;LET_DEF;LET_END_DEF;REVERSE;
interface_matrix;head_index;free_space_matrix;
MAP;replace_r_interface;int_of_op_comp;sign_cor_interface] THEN
REPEAT(POP_ASSUM MP_TAC) THEN REWRITE_TAC[GSYM ONE] 
THEN SIMP_TAC[GSYM MATRIX_MUL_ASSOC] THEN
SIMP_TAC[ MAT2X2_MUL;REAL_MUL_RZERO;REAL_MUL_LZERO;REAL_ADD_LID;REAL_ADD_RID;
REAL_MUL_LID;REAL_MUL_RID ] THEN 
SIMP_TAC[MAT2X2_EQ ] THEN CONV_TAC REAL_FIELD; ALL_TAC] THEN
CONJ_TAC THENL[REWRITE_TAC[DET_MAT2X2;fp_frl_matrix_xz] THEN
REPEAT(POP_ASSUM MP_TAC) THEN CONV_TAC REAL_FIELD; ALL_TAC] THEN
REWRITE_TAC[fp_frl_matrix_xz;mat2x2;VECTOR_2] THEN
REPEAT(POP_ASSUM MP_TAC) THEN CONV_TAC REAL_FIELD);;



(*----YZ--dimension--------*)


let fp_frl_cav_yz = new_definition
  ` fp_frl_cav_yz dy df nf : resonator =
    plane_r,[(&1,dy),(spherical_t (df/ &2));
            (nf,df),(spherical_t (--df/ &2))],(&1,dy),plane_r`;;

let fp_frl_yz_mat_u = new_definition 
  ` fp_frl_yz_mat_u dy df nf =
         mat2x2 (--(df * (-- &2 + nf) + &4 * dy * (-- &1 + nf)) / (df * nf))
         (((df + &2 * dy) * (df - &2 * dy * (-- &1 + nf))) / (df * nf))
         ((&4 - &4 * nf) / (df * nf))
         (--(df * (-- &2 + nf) + &4 * dy * (-- &1 + nf)) / (df * nf))`;;



let VALID_FP_FRL_YZ_CAVITY = prove
  (`!dy df nf.
    &0 < dy /\ &0 < df /\ &0 < nf  ==>
       is_valid_resonator  (fp_frl_cav_yz dy df nf)`,
  VALID_RESONATOR_TAC fp_frl_cav_yz);;

let VALID_RESONATOR def thm = prove (thm,
                               VALID_RESONATOR_TAC def);;

(*=========================================================================
EXAMPLE:

let RANGE_2_YZ_TIME = prove (
                      `d IN real_interval (#38 * #0.000001,#97 * #0.000001)
                       ==> is_stable_resonator (FP_YZ_RES d)`,
                       STABILITY_PROVE_TIME [FP_YZ_RES]) ;; 
===========================================================================*)


let TRACE_FP_FRL_YZ = prove 
(`! dy df nf .  &0 < dy /\ &0 < df /\ &0 < nf  ==> 
                 trace (fp_frl_yz_mat_u  dy df nf) =
           --(( &2* (df * (-- &2 + nf) + &4 * dy* (-- &1 + nf)))/(df* nf))`,
SIMP_TAC[TRACE_MAT2X2;fp_frl_yz_mat_u] THEN 
CONV_TAC REAL_FIELD);;

let VALID_RES = VALID_RESONATOR fp_frl_cav_yz `!dy df nf.
    &0 < dy /\ &0 < df /\ &0 < nf  ==>
       is_valid_resonator  (fp_frl_cav_yz dy df nf)`;;



let LEMMA_BASIC = REAL_FIELD `
! dy df nf.  &0 < dy /\ &0 < df /\ &0 < nf ==> 
(--(df * (-- &2 + nf) + &4 * dy * (-- &1 + nf)) / (df * nf) +
  --(df * (-- &2 + nf) + &4 * dy * (-- &1 + nf)) / (df * nf)) = 
 --(( &2* (df * (-- &2 + nf) + &4 * dy* (-- &1 + nf)))/(df* nf))`;;

let lemma1 = REAL_FIELD `&0 < dy /\ &0 < df /\ &0 < nf ==> 
              --((&2 * (df * (-- &2 + nf) + &4 * dy * 
             (-- &1 + nf))) / (df * nf)) / &2 =
              --(&1 - &2 / nf + (&4 * dy / df) * (&1 - &1 / nf))`;;

let lemma2 = REAL_FIELD `&0 < dy /\ &0 < df /\ &0 < nf ==> 
              --((&2 * (df * (-- &2 + nf) + &4 * dy * 
             (-- &1 + nf))) / (df * nf)) / &2 =
              --(&1 - &2 / nf + (&4 * dy / df) * (&1 - &1 / nf))`;;


let lemma3 = REAL_FIELD `&0 < dy /\ &0 < df /\ &0 < nf ==> 
               --((&2 * (df * (-- &2 + nf) + &4 * dy * 
              (-- &1 + nf))) / (df * nf)) / &2 =
               --(&1 - &2 / nf + (&4 * dy / df) * (&1 - &1 / nf))`;;


let common_defs = [system_composition;unfold_resonator_s;LIST_POW;
                   ONE;fp_frl_cav_yz;round_trip;APPEND;optical_components_shift;
                   LET_DEF;LET_END_DEF;REVERSE;
                   interface_matrix;head_index;free_space_matrix;MAP;
                   replace_r_interface;int_of_op_comp;
                   sign_cor_interface];;


let FP_FRL_YZ_MATRIX = prove (`
  !dy df nf dy. !dy df nf. &0 < dy /\ &0 < df /\ &0 < nf ==>
                system_composition (unfold_resonator_s 
                                    (fp_frl_cav_yz dy df nf) 1) = 
                fp_frl_yz_mat_u dy df nf  pow 2`,
REWRITE_TAC[fp_frl_yz_mat_u;MAT_POW2] THEN
REWRITE_TAC common_defs THEN
REPEAT(POP_ASSUM MP_TAC) THEN REWRITE_TAC[GSYM ONE] THEN 
SIMP_TAC[GSYM MATRIX_MUL_ASSOC] THEN
SIMP_TAC[MAT2X2_MUL;REAL_MUL_RZERO;REAL_MUL_LZERO;REAL_ADD_LID;
         REAL_ADD_RID;REAL_MUL_LID;REAL_MUL_RID] THEN 
SIMP_TAC[MAT2X2_EQ ] THEN 
SIMP_TAC[lemma1] THEN 
CONV_TAC REAL_FIELD);;





let STABLE_FP_FRL_YZ =  prove(` 
              !dy df nf. &0 < dy /\ &0 < df /\ &0 < nf /\  
              &0 < (&1 - &2/nf) + (&4*dy/df)*(&1 - &1/nf) /\
              (&1 - &2/nf) + (&4*dy/df)*(&1 - &1/nf)  < &1 ==> 
              is_stable_resonator(fp_frl_cav_yz dy df nf)`,


REPEAT STRIP_TAC THEN MP_REWRITE_TAC STABILITY_THEOREM_SYM THEN 
EXISTS_TAC(`fp_frl_yz_mat_u dy df nf  :real^2^2`) THEN 
CONJ_TAC THENL[ASM_SIMP_TAC[VALID_FP_FRL_YZ_CAVITY]; ALL_TAC] THEN
CONJ_TAC THENL[ASM_SIMP_TAC[FP_FRL_YZ_MATRIX]; ALL_TAC] THEN
CONJ_TAC THENL[REWRITE_TAC[DET_MAT2X2;fp_frl_yz_mat_u] THEN
REPEAT(POP_ASSUM MP_TAC) THEN
SIMP_TAC[lemma2] THEN 
 asm_real_prove ; ALL_TAC] THEN
REWRITE_TAC[fp_frl_yz_mat_u ;VECTOR_2;mat2x2 ] THEN
IMP_REWRITE_TAC[LEMMA_BASIC] THEN 

REPEAT(POP_ASSUM MP_TAC) THEN
SIMP_TAC[lemma3] THEN 
asm_real_prove);;


let STABILITY_FP_FRL = CONJ FP_FRL_STABLE_XZ STABLE_FP_FRL_YZ;;

 let TOTAL_STABILITY_FP_FRL = prove( `!R dx df nf.(~(R = &0) /\
          &0 < dx /\ &0 < dy /\
          &0 < df /\
          &0 < nf /\
          &0 < (&2 * dx + df / nf) / R /\
          (&2 * dx + df / nf) / R < &2 /\ 
          &0 < &1 - &2 / nf + (&4 * dy / df) * (&1 - &1 / nf) /\
          &1 - &2 / nf + (&4 * dy / df) * (&1 - &1 / nf) < &1) ==> 
          ((is_stable_resonator (fp_frl_xz_cavity R dx df nf)) /\ 
            is_stable_resonator (fp_frl_cav_yz dy df nf))`,
MESON_TAC[STABILITY_FP_FRL]);;



(*---------- Verification of the special cases taken ---------------
  -------------from the following paper-----------------------------

STABLE, HIGH-Q FABRY-PEROT RESONATORS WITH LONG CAVITY
BASED ON CURVED, ALL-SILICON, HIGH REFLECTANCE MIRRORS
[M. Malak, N. Pavy, F. Marty, Y.A. Peter A.Q. Liu and 
 T. Bourouina, IEEE MEMS 2011]

nf = 1.47
dx = dy
df = 125 (micro meter)
R  = 140 (micro meter)
--------------------------------------------------------------------*)


let STABILITY_PROVE t L = prove (t,
REWRITE_TAC (L@[real_interval;IN_ELIM_THM]) THEN 
 REPEAT STRIP_TAC THEN 
(MATCH_MP_TAC FP_FRL_STABLE_XZ ORELSE 
MATCH_MP_TAC STABLE_FP_FRL_YZ) THEN 
asm_real_prove);;

let STABILITY_PROVE2 t  = prove (t,
REWRITE_TAC ([real_interval;IN_ELIM_THM]) THEN 
 REPEAT STRIP_TAC THEN 
(MATCH_MP_TAC FP_FRL_STABLE_XZ ORELSE 
MATCH_MP_TAC STABLE_FP_FRL_YZ) THEN 
asm_real_prove);;

let STABILITY_PROVE_TIME L = time (REWRITE_TAC L THEN 
                             CONV_TAC STABILITY_PROVE2);;

let STABILITY_PROVE_TAC L = 
REWRITE_TAC (L@[real_interval;IN_ELIM_THM]) THEN 
REPEAT STRIP_TAC THEN 
(MATCH_MP_TAC FP_FRL_STABLE_XZ ORELSE 
 MATCH_MP_TAC STABLE_FP_FRL_YZ) THEN 
asm_real_prove;;





let FP_XZ_RES = define `  
            FP_XZ_RES d = fp_frl_xz_cavity  
                          (#140 * #0.000001) d  (#125.0 * #0.000001) (#1.47)`;;

let FP_YZ_RES = define `  FP_YZ_RES d  = fp_frl_cav_yz 
                          d (#125.0 * #0.000001) (#1.47)`;;


let STABILITY_PROVE_TAC g = prove (g ,
                             STABILITY_PROVE_TIME [FP_XZ_RES;FP_YZ_RES]);;


let RANGE_1_XZ = STABILITY_PROVE  `
                 (d IN real_interval (#27.5 * #0.000001,#35 * #0.000001))
                  ==> is_stable_resonator (FP_XZ_RES d)` [FP_XZ_RES];; 

let RANGE_2_XZ = STABILITY_PROVE  `
                 (d IN real_interval (#38 * #0.000001,#97 * #0.000001))
                  ==> is_stable_resonator (FP_XZ_RES d)` [FP_XZ_RES];; 


let RANGE_1_YZ = STABILITY_PROVE  `
                 (d IN real_interval (#97.5 * #0.000001,#132.9 * #0.000001))
                  ==> is_stable_resonator (FP_YZ_RES d)` [FP_YZ_RES];; 


let RANGE_2_YZM = STABILITY_PROVE `
                  (d IN real_interval (#38 * #0.000001,#97 * #0.000001))
                   ==> is_stable_resonator (FP_YZ_RES d)` [FP_YZ_RES];; 


let RANGE_2_XZM = STABILITY_PROVE `
                  (d IN real_interval (#38 * #0.000001,#97 * #0.000001))
                   ==> is_stable_resonator (FP_XZ_RES d)` [FP_XZ_RES];; 

(*----------------------VERIFICATION WITH TIME ---------------------------------*)



let RANGE_1_XZ_TIME = prove (
                      `d IN real_interval (#27.5 * #0.000001,#35 * #0.000001)
                       ==> is_stable_resonator (FP_XZ_RES d)`,
                       STABILITY_PROVE_TIME [FP_XZ_RES]);; 

let RANGE_2_XZ_TIME = prove (
                      `d IN real_interval (#38 * #0.000001,#97 * #0.000001)
                       ==> is_stable_resonator (FP_XZ_RES d)`,
                       STABILITY_PROVE_TIME [FP_XZ_RES]);; 


let RANGE_1_YZ_TIME = prove (
                      `d IN real_interval (#97.5 * #0.000001,#132.9 * #0.000001)
                       ==> is_stable_resonator (FP_YZ_RES d)`,
                       STABILITY_PROVE_TIME [FP_YZ_RES]) ;; 


let RANGE_2_YZ_TIME = prove (
                      `d IN real_interval (#38 * #0.000001,#97 * #0.000001)
                       ==> is_stable_resonator (FP_YZ_RES d)`,
                       STABILITY_PROVE_TIME [FP_YZ_RES]) ;; 
    
let RANGE_2_XZM_TIME = prove (
                      `d IN real_interval (#38 * #0.000001,#97 * #0.000001)
                       ==> is_stable_resonator (FP_XZ_RES d)`,
                       STABILITY_PROVE_TIME [FP_XZ_RES]) ;; 
