(* ========================================================================= *)
(*                                                                           *)
(*                Ring Resonator Chaos Verification                          *)
(*                                                                           *)
(*        (c) Copyright, Umair Siddique, 2012 - 2015                         *)
(*                       Hardware Verification Group,                        *)
(*                       Concordia University                                *)
(*                                                                           *)
(*     Contact: <muh_sidd@ece.concordia.ca>                                  *)
(*                                                                           *)
(* Last update: Sep  2015                                                    *)
(*                                                                           *)
(* ========================================================================= *)

needs "resonator.ml";;

let ring_reso = new_definition `
    ring_reso a b c d L n :resonator = 
    (plane_r),[(n, L / &2),(unknown a b c d); 
               (n,L / &2),plane_r],
               (n,L), pcm`;;


let ring_matrix  = new_definition  
   `ring_matrix a b c d L  = mat2x2 
                       (a + (&3 * c * L) / &2 )  
                       (b + (&3 * L) / &4 * (&2 * a + &3 * c * L + &2 * d))
                       (-- c) 
                       (-- (&3 * c * L) / &2 - d) `;;

let RR_HALF_ROUND_TRIP_MAT = prove (
  `! a b c d L. 
     system_composition ((half_round_trip (ring_reso a b c d L n) (n, L))) = 
     ring_matrix a b c d L`,

REPEAT GEN_TAC THEN 
REWRITE_TAC[ring_matrix;half_round_trip;system_composition;ring_reso;
            interface_matrix;head_index;free_space_matrix;APPEND] THEN 
SIMP_TAC[MAT2X2_MUL] THEN 
SIMP_TAC[MAT2X2_EQ] THEN 
CONV_TAC REAL_FIELD);;


let RR_DUFFING_CHAOS_VERIFICATION = prove(
`!n L. (&0 < n /\ &0 <= L) /\ 
       (!theta0. mat2x2 a b c d = 
                 mat2x2 (-- (&3 * beta * L) / &2)
                        (&1 / &4 * ( &4  + &6 * alpha * L + 
                         &9 * beta * L * L -  &6  * L * theta0 * theta0)) 
                        (beta) 
                        (-- alpha - &3 / &2 * (L * beta) + theta0 * theta0)) ==>
chaotic_resonator 
     (ring_reso a b c d L n)
     (\ (y0,theta0) (y1,theta1).
        duffin_map (y0,theta0) (y1,theta1) alpha beta) (n, L)`,

REPEAT STRIP_TAC THEN 
MATCH_MP_TAC DUFFIN_CHAOTIC_GEN_CODITIONS THEN 
REPEAT STRIP_TAC THENL[
REPEAT (POP_ASSUM MP_TAC) THEN
SIMP_TAC[ RR_HALF_ROUND_TRIP_MAT] THEN 
SIMP_TAC [ring_matrix; duffin_map_matrix] THEN  
SIMP_TAC[MAT2X2_EQ] THEN 
CONV_TAC REAL_FIELD;ALL_TAC]  THEN 
SIMP_TAC[half_round_trip;is_valid_optical_system;ALL;APPEND;ring_reso;
is_valid_optical_component;is_valid_free_space; is_valid_interface] THEN 
REPEAT(POP_ASSUM MP_TAC) THEN CONV_TAC REAL_FIELD);;


